/* -------------------------------------------------------------------------- *
 * file:         scanconfig.c                                                 *
 * purpose:      nessus server login and plugin retrieval                     *
 * ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <cgic.h>

#include "nessuswc.h"

int cgiMain() {

   static char 	title[] = "Scanner Configuration";
   char		nessusd_ip[16];
   int		nessusd_port;
   char		nessusd_user[81];
   char		nessusd_pass[81];
   char		nessusd_cert[81];
   char		nessusd_encr[81];
   int		plugs_counter = 0;
   int		prefs_counter = 0;
   int		rules_counter = 0;
   SSL		*ssl;
   int 		i = 0;

/* -------------------------------------------------------------------------- *
 * check if we got all information to make a nessus server connection         *
 * ---------------------------------------------------------------------------*/

  if ( cgiFormString("nessusd-ip", nessusd_ip, sizeof(nessusd_ip))
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus server IP address.");

  if ( cgiFormInteger("nessusd-port", &nessusd_port, NESSUSD_PORT)
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus server port number.");

  if ( nessusd_port <= 0 || nessusd_port > 65535 )
    int_error("Error Nessus server port number not in a valid port range.");

  if ( cgiFormString("nessusd-encr", nessusd_encr, sizeof(nessusd_encr))
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus server protocol encryption type.");

  if ( cgiFormString("nessusd-user", nessusd_user, sizeof(nessusd_user))
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus remote user name.");

  if ( cgiFormString("nessusd-pass", nessusd_pass, sizeof(nessusd_pass))
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus remote user password.");

  if ( cgiFormString("nessusd-cert", nessusd_cert, sizeof(nessusd_cert))
                                                   != cgiFormSuccess )
    int_error("Error retrieving Nessus client certificate name.");
  
/* -------------------------------------------------------------------------- *
 * make a TCP connection to the Nessus Server.                                *
 * ---------------------------------------------------------------------------*/

  ssl = nessus_connect(nessusd_ip, nessusd_port, nessusd_encr, nessusd_cert);

/* -------------------------------------------------------------------------- *
 * Handle the login to the Nessus Server.                                     *
 * ---------------------------------------------------------------------------*/

  nessus_login(ssl, nessusd_user, nessusd_pass);

/* -------------------------------------------------------------------------- *
 * Get the list of plugins from the Nessus Server and                         *
 * create the list of categories.                                             *
 * ---------------------------------------------------------------------------*/

  plugs_counter = nessus_getplugs(ssl);
  if (plugs_counter == 0)
     int_error("Could not receive plugins from Nessus server.");

  nessus_getgroups(plugs_counter);

/* -------------------------------------------------------------------------- *
 * Get the preferences list from the Nessus Server.                           *
 * ---------------------------------------------------------------------------*/

  prefs_counter = nessus_getprefs(ssl);
  if (prefs_counter == 0)
     int_error("Could not receive preferences from Nessus server.");

/* -------------------------------------------------------------------------- *
 * Get the rules list from the Nessus Server.                                 *
 * ---------------------------------------------------------------------------*/

  rules_counter = nessus_getrules(ssl);
  /* it isn't unusual to have no rules so the count can be zero. */

/* -------------------------------------------------------------------------- *
 * start the html output                                                      *
 * ---------------------------------------------------------------------------*/

   pagehead(title, NULL, cgiOut);

/* -------------------------------------------------------------------------- *
 * start the form output                                                      *
 * ---------------------------------------------------------------------------*/

   fprintf(cgiOut, "<p><center>");
   fprintf(cgiOut, "<form action=\"scanprocess.cgi\" method=\"post\">");

   /* forward the login information to the next cgi */
   fprintf(cgiOut, "<input type=hidden name=nessusd-ip value=%s>", nessusd_ip);
   fprintf(cgiOut, "<input type=hidden name=nessusd-port value=%d>", nessusd_port);
   fprintf(cgiOut, "<input type=hidden name=nessusd-encr value=%s>", nessusd_encr);
   fprintf(cgiOut, "<input type=hidden name=nessusd-user value=%s>", nessusd_user);
   fprintf(cgiOut, "<input type=hidden name=nessusd-pass value=%s>", nessusd_pass);
   fprintf(cgiOut, "<input type=hidden name=nessusd-cert value=%s>", nessusd_cert);

   fprintf(cgiOut, "<table border=\"1\" cellspacing=\"0\" ");
   fprintf(cgiOut, "cellpadding=\"2\" width=\"540\">");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" colspan=4>");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "Nessus Scan Target IP and Netmask");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<td bgcolor=CFCFCF align=\"center\" bordercolor=CFCFCF>");
   fprintf(cgiOut, "IP Address:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td bgcolor=FFFFFF align=\"center\">");
   fprintf(cgiOut, "<input type=text name=t-ip size=15 maxlength=15 value=%s>", DEFAULT_TARGET_IP);
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td bgcolor=CFCFCF align=\"center\" bordercolor=CFCFCF>");
   fprintf(cgiOut, "Netmask:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td bgcolor=FFFFFF align=\"center\">");
   fprintf(cgiOut, "<input type=text name=t-mask size=15 maxlength=15 value=255.255.255.255>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" colspan=4>");
   fprintf(cgiOut, "<input type=\"submit\" value=\"Start Scan\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "</table><p>");

   fprintf(cgiOut, "<table border=\"1\" cellspacing=\"0\" ");
   fprintf(cgiOut, "cellpadding=\"2\" width=\"540\">");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" colspan=6>");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "Nessus Scanner Plugin Family List");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");

   for(i=0; i<famly_counter; i++) {

     fprintf(cgiOut, "<td bgcolor=CFCFCF align=\"center\" bordercolor=CFCFCF>");
     fprintf(cgiOut, "<input type=checkbox name=\"%s\">", famlylist[i].name);
     fprintf(cgiOut, "</td>");
     fprintf(cgiOut, "<td bgcolor=FFFFFF>");
     fprintf(cgiOut, "%s", famlylist[i].name);
     fprintf(cgiOut, "</td>");
     fprintf(cgiOut, "<td bgcolor=FFFFFF align=\"right\">");
     fprintf(cgiOut, "%d", famlylist[i].plugscount);
     fprintf(cgiOut, "</td>");

    /* we want to display 2 columns of 3 cells (checkbox | Family Name | *
     * Plugin Count) to shorten the length of the overall Family list.   */
     if( i == 1 || (i % 2) != 0 ) fprintf(cgiOut, "</tr><tr>\n");

   }

   /* if famly_counter is a uneven number we miss a cell at the last row *
    * so we better insert a "dummy".                                     */
   if( (famly_counter % 2) != 0 ) {
      fprintf(cgiOut, "<td bgcolor=CFCFCF bordercolor=CFCFCF>&nbsp;</td>\n");
      fprintf(cgiOut, "<td bgcolor=FFFFFF colspan=2>&nbsp;</td>\n");
   }
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" colspan=6>");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "Total: %d Plugins", plugs_counter );
   fprintf(cgiOut, " in %d Families.", famly_counter );
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "</table></form>\n");

   fprintf(cgiOut, "<p><center>");
   fprintf(cgiOut, "<table border=\"1\" cellspacing=\"0\" ");
   fprintf(cgiOut, "cellpadding=\"2\" width=\"540\">");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" colspan=2>");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "NessusWC Client");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td align=\"center\" bgcolor=\"#CFCFCF\"");
   fprintf(cgiOut, "bordercolor=\"#CFCFCF\" width=180>");
   fprintf(cgiOut, "Version:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td align=\"center\"bgcolor=\"#FFFFFF\">");
   fprintf(cgiOut, "%s\n", CLIENT_SW_VERSION);
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>\n");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td align=\"center\" bgcolor=CFCFCF bordercolor=CFCFCF>");
   fprintf(cgiOut, "Copyright:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td align=\"center\"bgcolor=\"#FFFFFF\">");
   fprintf(cgiOut, "%s\n", CLIENT_COPYRIGHTS);
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>");
   fprintf(cgiOut, "<td align=\"center\" bgcolor=CFCFCF bordercolor=CFCFCF>");
   fprintf(cgiOut, "Status:");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td align=\"center\"bgcolor=\"#FFFFFF\">");
   fprintf(cgiOut, "<b>%s</b> login to server <b>%s</b> successful.",
                    nessusd_user, nessusd_ip);
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" colspan=2>");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "End");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "</table>\n");

/* -------------------------------------------------------------------------- *
 * end the html output                                                        *
 * ---------------------------------------------------------------------------*/

   fprintf(cgiOut, "</BODY></HTML>\n");
   return(0);
}
