/* -------------------------------------------------------------------------- *
 * file:         scanresults.c                                                *
 * purpose:      builds a list of session html files in the results directory *
 * ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <cgic.h>

#include "nessuswc.h"

#define _XOPEN_SOURCE

int cgiMain() {

  static char 	title[] = "List of Scan Results";
  char		targetdec_ip[16] = "";
  char		targetdot_ip[16] = "";
  char		timestamp[16] = "";
  char		scantimestr[33] = "";
  char		*buf_ptr = NULL;
  DIR		*resultsdir;
  struct	dirent *resultsdir_files;
  struct	in_addr target_ip;
  struct	tm scantime;
  char		resultsfilestr[225];
  int pagenumber = 1;
  int rescounter = 0;
  int tempcounter = 0;
  int pagecounter = 0;
  int dispcounter = 0;
  int fposcounter = 0;
  int dispmaxlines = 0;

/* -------------------------------------------------------------------------- *
 * See if we are in a initial or subsequent cgi call                          *
 * ---------------------------------------------------------------------------*/
  cgiFormInteger("resfound", &rescounter, 0)  == cgiFormSuccess;
  cgiFormInteger("pagecount", &pagecounter, 0) == cgiFormSuccess;
  cgiFormInteger("page", &pagenumber, 1) == cgiFormSuccess;

  if(rescounter == 0) {
    if (! (resultsdir = opendir(RESULTS_DIR)))
       int_error("Error opening scan result directory");
    for(;;) {
      resultsdir_files = readdir(resultsdir);
      if(! resultsdir_files) break;
      if( strstr(resultsdir_files->d_name, ".htm")) {
        rescounter++;
        tempcounter++;
        if(tempcounter == MAXRESDISPLAY) {
           pagecounter++;
           tempcounter=0;
        }
      }
    }
    closedir(resultsdir);
    if(tempcounter > 0) pagecounter++;
  }

/* -------------------------------------------------------------------------- *
 * now we know how many results we have in total and we can build the page(s).*
 * For every 25 results we start a new page and cycle through by calling      *
 * ourself with the requested results range.                                  *
 * ---------------------------------------------------------------------------*/

   if(rescounter <= MAXRESDISPLAY) {
      dispmaxlines = rescounter;
      tempcounter = 0;
   }

   else if(pagenumber == pagecounter &&
            ( pagecounter * MAXRESDISPLAY) - rescounter != 0) {

      tempcounter = (pagecounter * MAXRESDISPLAY) - MAXRESDISPLAY;
      dispmaxlines = rescounter - ((pagecounter-1) * MAXRESDISPLAY);
   }

   else {

      tempcounter = (pagenumber * MAXRESDISPLAY) - MAXRESDISPLAY;
      dispmaxlines = MAXRESDISPLAY;
   }

/* -------------------------------------------------------------------------- *
 * start the html output                                                      *
 * ---------------------------------------------------------------------------*/

   pagehead(title, NULL, cgiOut);

/* -------------------------------------------------------------------------- *
 * start the form output                                                      *
 * ---------------------------------------------------------------------------*/

   fprintf(cgiOut, "<p><center>");
   fprintf(cgiOut, "<table border=\"1\" cellspacing=\"0\" ");
   fprintf(cgiOut, "cellpadding=\"2\" width=\"540\">");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" width=\"5\">");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "#</font>");
   fprintf(cgiOut, "</td>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" width=\"235\">");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "Scan Session");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" width=\"185\">");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "Scan Date");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" width=\"80\">");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "Scan Target");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" width=\"35\">");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "View</font>");
   fprintf(cgiOut, "</td>\n");

   resultsdir = opendir(RESULTS_DIR);
   // fast forward to the start cert of this page
   // printf("open dir succeeded.\n");
   for(;;) {
      resultsdir_files = readdir(resultsdir);
      // printf("%s%s\n", resultsdir_files->d_name, "filename\n");
      // printf("%d %s\n", fposcounter, "fposcounter.\n");
      if(! resultsdir_files) break;
      if( strstr(resultsdir_files->d_name, ".htm")) {
         if(fposcounter == tempcounter) break;
         fposcounter++;
      }
   }

  for(dispcounter=0; dispcounter < dispmaxlines; dispcounter++) {
    /* dis-assemble the session id into its time and ip values */
    if(strstr(resultsdir_files->d_name, ".htm")) {
      strncpy(resultsfilestr, resultsdir_files->d_name, sizeof(resultsfilestr));

      /* split the filename */
      buf_ptr = strchr(resultsfilestr, '-') + 1;
      strncpy(timestamp, buf_ptr, strcspn(buf_ptr, "_"));
      buf_ptr = strchr(resultsfilestr, '_') + 1;
      strncpy(targetdec_ip, buf_ptr, sizeof(targetdec_ip));
      /* here we blend out the .htm extension in targetdec_ip */
      *strchr(targetdec_ip, '.') = '\0';

      /* convert the decimal IP back into dotted format */
      target_ip.s_addr=atoi(targetdec_ip);
      strncpy(targetdot_ip, inet_ntoa(target_ip), sizeof(targetdot_ip));

      /* convert the timestamp into a human readable string */
      strptime(timestamp, "%s", &scantime);
      strftime(scantimestr, sizeof(scantimestr), "%B, %d. %Y %T",
                                                             &scantime);
    }

     resultsdir_files = readdir(resultsdir);
     tempcounter++;

     fprintf(cgiOut, "<tr>");
     fprintf(cgiOut, "<td bgcolor=\"000080\" align=\"center\">");
     fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"FFFFFF\">");
     fprintf(cgiOut, "%d", tempcounter);
     fprintf(cgiOut, "</font>");
     fprintf(cgiOut, "</td>\n");
     fprintf(cgiOut, "<td bgcolor=\"FFFFFF\" nowrap>");
     fprintf(cgiOut, "<font size=\"2\" face=\"Arial\">");
     fprintf(cgiOut, "%s\n", resultsfilestr);
     fprintf(cgiOut, "</font>");
     fprintf(cgiOut, "</td>\n");
     fprintf(cgiOut, "<td bgcolor=\"FFFFFF\" align=center nowrap>");
     fprintf(cgiOut, "<font size=\"2\" face=\"Arial\">");
     fprintf(cgiOut, "%s\n", scantimestr);
     fprintf(cgiOut, "</font>");
     fprintf(cgiOut, "</td>\n");
     fprintf(cgiOut, "<td bgcolor=\"FFFFFF\" align=right nowrap>");
     fprintf(cgiOut, "<font size=\"2\" face=\"Arial\">");
     fprintf(cgiOut, "%s\n", targetdot_ip);
     fprintf(cgiOut, "</font>");
     fprintf(cgiOut, "</td>\n");
     fprintf(cgiOut, "<form method=get action=../results/%s>", resultsfilestr);
     fprintf(cgiOut, "<td bgcolor=\"000080\">");
     fprintf(cgiOut, "<input type=\"submit\" value=\"View\">");
     fprintf(cgiOut, "</td>\n");
     fprintf(cgiOut, "</form>");
     fprintf(cgiOut, "</tr>");
   }
   closedir(resultsdir);

   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" ");
   fprintf(cgiOut, "colspan=\"6\">");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "End");
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</tr>");
   fprintf(cgiOut, "</table>\n");
   fprintf(cgiOut, "<p><center>");
   fprintf(cgiOut, "<table border=\"1\" cellspacing=\"0\" ");
   fprintf(cgiOut, "cellpadding=\"2\" width=\"540\">");
   fprintf(cgiOut, "<tr>\n");
   fprintf(cgiOut, "<td bgcolor=\"#000080\" align=\"center\" width=380>");
   fprintf(cgiOut, "<font size=\"2\" face=\"Arial\" color=\"#FFFFFF\">");
   fprintf(cgiOut, "Total # of results: %d | ", rescounter);
   fprintf(cgiOut, "Page %d of %d", pagenumber, pagecounter);
   fprintf(cgiOut, "</font>");
   fprintf(cgiOut, "</td>\n");
   // goto page 1
   fprintf(cgiOut, "<form action=\"scanresults.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td width=5>");
   fprintf(cgiOut, "<input type=\"submit\" value=\"<<\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>");
   // goto page before
   fprintf(cgiOut, "<form action=\"scanresults.cgi\" method=\"post\">");
   fprintf(cgiOut, "<input type=\"hidden\" name=\"rescounter\" ");
   fprintf(cgiOut, "value=\"");
   fprintf(cgiOut, "%d", rescounter);
   fprintf(cgiOut, "\">\n");
   fprintf(cgiOut, "<input type=\"hidden\" name=\"pagecounter\" ");
   fprintf(cgiOut, "value=\"");
   fprintf(cgiOut, "%d", pagecounter);
   fprintf(cgiOut, "\">\n");
   fprintf(cgiOut, "<input type=\"hidden\" name=\"page\" ");
   fprintf(cgiOut, "value=\"");
   tempcounter = 0;
   if(pagenumber > 1) tempcounter = pagenumber - 1;
   else tempcounter = 1;
   fprintf(cgiOut, "%d", tempcounter);
   fprintf(cgiOut, "\">\n");
   fprintf(cgiOut, "<td width=5>");
   fprintf(cgiOut, "<input type=\"submit\" value=\"<\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>");
   // goto page after
   fprintf(cgiOut, "<form action=\"scanresults.cgi\" method=\"post\">");
   fprintf(cgiOut, "<input type=\"hidden\" name=\"rescounter\" ");
   fprintf(cgiOut, "value=\"");
   fprintf(cgiOut, "%d", rescounter);
   fprintf(cgiOut, "\">\n");
   fprintf(cgiOut, "<input type=\"hidden\" name=\"pagecounter\" ");
   fprintf(cgiOut, "value=\"");
   fprintf(cgiOut, "%d", pagecounter);
   fprintf(cgiOut, "\">\n");
   fprintf(cgiOut, "<input type=\"hidden\" name=\"page\" ");
   fprintf(cgiOut, "value=\"");
   tempcounter = 0;
   if(pagecounter > pagenumber) tempcounter = pagenumber + 1;
   else tempcounter = pagecounter;
   fprintf(cgiOut, "%d", tempcounter);
   fprintf(cgiOut, "\">\n");
   fprintf(cgiOut, "<td width=5>");
   fprintf(cgiOut, "<input type=\"submit\" value=\">\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>");
   // goto last page
   fprintf(cgiOut, "<form action=\"scanresults.cgi\" method=\"post\">");
   fprintf(cgiOut, "<input type=\"hidden\" name=\"rescounter\" ");
   fprintf(cgiOut, "value=\"");
   fprintf(cgiOut, "%d", rescounter);
   fprintf(cgiOut, "\">\n");
   fprintf(cgiOut, "<input type=\"hidden\" name=\"pagecounter\" ");
   fprintf(cgiOut, "value=\"");
   fprintf(cgiOut, "%d", pagecounter);
   fprintf(cgiOut, "\">\n");
   fprintf(cgiOut, "<input type=\"hidden\" name=\"page\" ");
   fprintf(cgiOut, "value=\"");
   fprintf(cgiOut, "%d", pagecounter);
   fprintf(cgiOut, "\">\n");
   fprintf(cgiOut, "<td width=5>");
   fprintf(cgiOut, "<input type=\"submit\" value=\">>\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>");
   fprintf(cgiOut, "<form action=\"scanresults.cgi\" method=\"post\">");
   fprintf(cgiOut, "<td width=10 nowrap>");
   fprintf(cgiOut, "<input type=\"hidden\" name=\"rescounter\" ");
   fprintf(cgiOut, "value=\"");
   fprintf(cgiOut, "%d", rescounter);
   fprintf(cgiOut, "\">\n");
   fprintf(cgiOut, "<input type=\"hidden\" name=\"pagecounter\" ");
   fprintf(cgiOut, "value=\"");
   fprintf(cgiOut, "%d", pagecounter);
   fprintf(cgiOut, "\">\n");
   fprintf(cgiOut, "<input type=\"submit\" value=\"Goto\">");
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "<td width=10 nowrap>");
   fprintf(cgiOut, "<input type=\"text\" name=\"page\" size=4 value=");
   fprintf(cgiOut, "%d>\n", pagecounter);
   fprintf(cgiOut, "</td>");
   fprintf(cgiOut, "</form>");
   fprintf(cgiOut, "</tr>\n");
   fprintf(cgiOut, "</table>\n");

/* -------------------------------------------------------------------------- *
 * end the html output                                                        *
 * ---------------------------------------------------------------------------*/

   fprintf(cgiOut, "</BODY></HTML>\n");
   return(0);
}
